BEGIN TRANSACTION;

--全量数据：创建记录表
CREATE TABLE objects_list_kdk(
	id INTEGER PRIMARY KEY AUTOINCREMENT,
	path TEXT NOT NULL,
	hash TEXT NOT NULL,
	type TEXT NOT NULL CHECK(type in ('program','library','kmodule','script','other')),
	exectl TEXT NOT NULL CHECK(exectl in ('original', 'verified', 'kysoft', 'trusted', 'parent', 'script', 'unknown')),
	filepro TEXT NOT NULL CHECK(filepro in ('none', 'readonly')),
	rmmodpro INTEGER NOT NULL DEFAULT 0 CHECK(rmmodpro in (0,1)),
	status INTEGER NOT NULL DEFAULT 0, --执行控制或内核模块保护状态：0 正常，1 已篡改，2不存在，3 已解除
	sm_status INTEGER NOT NULL DEFAULT 0, --数据源类型： 0 常规系统数据 1 涉密安全套件数据 2 通用安全套件数据
	data_source_name TEXT NOT NULL DEFAULT '' --数据源名称
);

--全量数据：执行控制功能白名单视图
CREATE VIEW exectl_whlist_kdk AS
SELECT id, path, hash, type, exectl, filepro, rmmodpro, status, sm_status, data_source_name from objects_list_kdk where type in ('program', 'library', 'script') and status != 2;

--全量数据：文件保护功能视图
CREATE VIEW filepro_whlist_kdk AS
SELECT id, path, hash, type, exectl, filepro, rmmodpro, status, sm_status ,data_source_name from objects_list_kdk where filepro = 'readonly';

--全量数据：内核模块保护功能白名单视图
CREATE VIEW kmodpro_whlist_kdk AS
SELECT id, path, hash, type, exectl, filepro, rmmodpro, status, sm_status ,data_source_name from objects_list_kdk where type in ('kmodule') and status != 2;

--通用功能
CREATE TABLE relabel_status(
	relabel INTEGER NOT NULL DEFAULT 0 --标记初始化标识： 0 全盘初始化，1 按照白名单初始化，2 重新初始化执行控制白名单
);

--全量数据
CREATE TABLE IF NOT EXISTS kmod_list_kdk(
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    kmod_name TEXT NOT NULL,
    status INTEGER NOT NULL DEFAULT 0 CHECK(status in (0,1)),
    sm_status INTEGER NOT NULL DEFAULT 0, --数据源类型： 0 常规系统数据 1 涉密安全套件数据 2 通用安全套件数据
    ctime INTEGER  NOT NULL DEFAULT (strftime('%s','now')),
    data_source_name TEXT NOT NULL DEFAULT '' --数据源名称
);

--通用功能
CREATE VIEW IF NOT EXISTS kmod_whlist AS
SELECT id, kmod_name, status, ctime from kmod_list_kdk where sm_status = 0;

--通用功能
CREATE TRIGGER IF NOT EXISTS kmod_whlist_delete
INSTEAD OF DELETE ON kmod_whlist
BEGIN
	UPDATE kmod_list_kdk SET status = 0, ctime = strftime('%s','now') WHERE kmod_name=OLD.kmod_name and sm_status = 0;
END;

--通用功能
CREATE TRIGGER IF NOT EXISTS kmod_whlist_insert_exists
INSTEAD OF INSERT ON kmod_whlist WHEN EXISTS(select 1 from kmod_list_kdk where kmod_name=NEW.kmod_name and sm_status = 0)
BEGIN
	UPDATE kmod_list_kdk SET status = NEW.status, ctime = strftime('%s','now') WHERE kmod_name=NEW.kmod_name and sm_status = 0;
END;

--通用功能
CREATE TRIGGER IF NOT EXISTS kmod_whlist_insert_not_exists
INSTEAD OF INSERT ON kmod_whlist WHEN NOT EXISTS(select 1 from kmod_list_kdk where kmod_name=NEW.kmod_name and sm_status = 0)
BEGIN
    INSERT INTO kmod_list_kdk(kmod_name, status) VALUES(NEW.kmod_name, NEW.status);
END;

--全量数据
CREATE VIEW IF NOT EXISTS kmod_whlist_kdk AS
SELECT id, kmod_name, status, sm_status, ctime, data_source_name from kmod_list_kdk;

--全量数据
CREATE TRIGGER IF NOT EXISTS kmod_whlist_kdk_delete
INSTEAD OF DELETE ON kmod_whlist_kdk
BEGIN
	UPDATE kmod_list_kdk SET status = 0, ctime = strftime('%s','now') WHERE kmod_name=OLD.kmod_name and sm_status=OLD.sm_status and (data_source_name=OLD.data_source_name or data_source_name='');
END;

--全量数据
CREATE TRIGGER IF NOT EXISTS kmod_whlist_kdk_insert_exists
INSTEAD OF INSERT ON kmod_whlist_kdk WHEN EXISTS(select 1 from kmod_list_kdk where kmod_name=NEW.kmod_name and sm_status=NEW.sm_status and (data_source_name=NEW.data_source_name or data_source_name=''))
BEGIN
	UPDATE kmod_list_kdk SET status = NEW.status, data_source_name=NEW.data_source_name, ctime = strftime('%s','now') WHERE kmod_name=NEW.kmod_name and sm_status=NEW.sm_status and (data_source_name=NEW.data_source_name or data_source_name='');
END;

--全量数据
CREATE TRIGGER IF NOT EXISTS kmod_whlist_kdk_insert_not_exists
INSTEAD OF INSERT ON kmod_whlist_kdk WHEN NOT EXISTS(select 1 from kmod_list_kdk where kmod_name=NEW.kmod_name and sm_status=NEW.sm_status and (data_source_name=NEW.data_source_name or data_source_name=''))
BEGIN
    INSERT INTO kmod_list_kdk(kmod_name, status, sm_status, data_source_name) VALUES(NEW.kmod_name, NEW.status, NEW.sm_status, NEW.data_source_name);
END;

--通用功能：
DELETE from relabel_status;
INSERT into relabel_status(relabel) values(0);

--通用功能：创建记录视图
CREATE VIEW objects_list AS
SELECT id, path, hash, type, exectl, filepro, rmmodpro, status from objects_list_kdk where sm_status = 0;

--通用功能：执行控制功能白名单视图
CREATE VIEW exectl_whlist AS
SELECT id, path, hash, type, exectl, filepro, rmmodpro, status from objects_list where type in ('program', 'library', 'script') and status != 2;

--通用功能：文件保护功能视图
CREATE VIEW filepro_whlist AS
SELECT id, path, hash, type, exectl, filepro, rmmodpro, status from objects_list where filepro = 'readonly';

--通用功能：内核模块保护功能白名单视图
CREATE VIEW kmodpro_whlist AS
SELECT id, path, hash, type, exectl, filepro, rmmodpro, status from objects_list where type in ('kmodule') and status != 2;

COMMIT;

--通用功能：记录表插入不存在的记录
CREATE TRIGGER objects_list_insert_not_exists
INSTEAD OF INSERT ON objects_list WHEN NOT EXISTS(select 1 from objects_list_kdk where path=NEW.path and sm_status = 0)
BEGIN
	INSERT INTO objects_list_kdk(path, hash, type, exectl, filepro, rmmodpro, status, sm_status, data_source_name) values(NEW.path, NEW.hash, NEW.type, NEW.exectl, NEW.filepro, NEW.rmmodpro, NEW.status, 0, "");
END;

--通用功能：记录表插入存在的记录
CREATE TRIGGER objects_list_insert_exists
INSTEAD OF INSERT ON objects_list WHEN EXISTS(select 1 from objects_list_kdk where path=NEW.path and sm_status = 0)
BEGIN
	UPDATE objects_list_kdk SET hash = NEW.hash, type = NEW.type, exectl = NEW.exectl, filepro = NEW.filepro, rmmodpro = NEW.rmmodpro, status = NEW.status WHERE path=NEW.path and sm_status = 0;
END;

--通用功能：记录表删除记录
CREATE TRIGGER objects_list_delete
INSTEAD OF DELETE ON objects_list
BEGIN
	DELETE FROM objects_list_kdk where path = OLD.path and sm_status = 0;
END;

--通用功能：记录表更新记录
CREATE TRIGGER objects_list_update
INSTEAD OF UPDATE ON objects_list
BEGIN
	UPDATE objects_list_kdk SET hash = NEW.hash, type = NEW.type, exectl = NEW.exectl, filepro = NEW.filepro, rmmodpro = NEW.rmmodpro, status = NEW.status WHERE path=NEW.path and sm_status = 0;
END;

--全量数据：记录更新触发器
CREATE TRIGGER objects_list_kdk_delete AFTER UPDATE OF filepro
ON objects_list_kdk
BEGIN
	DELETE FROM objects_list_kdk where id = new.id and filepro = 'none' and type = 'other';
END;

--全量数据：执行控制功能白名单视图删除触发器，采用修改exectl字段值为'unknown'替换
CREATE TRIGGER exectl_whlist_kdk_delete
INSTEAD OF DELETE ON exectl_whlist_kdk
BEGIN
	UPDATE objects_list_kdk SET exectl = 'unknown', status = 3 WHERE path=OLD.path and sm_status=OLD.sm_status and (data_source_name=OLD.data_source_name or data_source_name='');
END;

--全量数据：执行控制功能白名单视图插入触发器，当记录已存在时，修改已存在记录exectl为新exectl状态
CREATE TRIGGER exectl_whlist_kdk_insert_exists
INSTEAD OF INSERT ON exectl_whlist_kdk WHEN EXISTS(select 1 from objects_list_kdk where path=NEW.path and sm_status=NEW.sm_status and (data_source_name=NEW.data_source_name or data_source_name=''))
BEGIN
	UPDATE objects_list_kdk SET exectl = NEW.exectl, hash = NEW.hash, status = 0, data_source_name=NEW.data_source_name WHERE path=NEW.path and sm_status=NEW.sm_status and (data_source_name=NEW.data_source_name or data_source_name='');
END;

--全量数据：执行控制功能白名单视图插入触发器，当记录不存在时，在表中插入记录
CREATE TRIGGER exectl_whlist_kdk_insert_not_exists
INSTEAD OF INSERT ON exectl_whlist_kdk WHEN NOT EXISTS(select 1 from objects_list_kdk where path=NEW.path and sm_status=NEW.sm_status and (data_source_name=NEW.data_source_name or data_source_name=''))
BEGIN
	INSERT INTO objects_list_kdk(path, hash, type, exectl, filepro, rmmodpro, status, sm_status, data_source_name) values(NEW.path, NEW.hash, NEW.type, NEW.exectl, 'none', 0, 0, NEW.sm_status, NEW.data_source_name);
END;

--全量数据：文件保护功能视图删除触发器，采用修改filepro字段值为none替换
CREATE TRIGGER filepro_whlist_kdk_delete
INSTEAD OF DELETE ON filepro_whlist_kdk
BEGIN
	UPDATE objects_list_kdk SET filepro = 'none' WHERE path=OLD.path and sm_status=OLD.sm_status and (data_source_name=OLD.data_source_name or data_source_name='');	
END;

--全量数据：文件保护功能白名单视图插入触发器
CREATE TRIGGER filepro_whlist_kdk_insert_exists
INSTEAD OF INSERT ON filepro_whlist_kdk WHEN EXISTS(select 1 from objects_list_kdk where path=NEW.path and sm_status=NEW.sm_status and (data_source_name=NEW.data_source_name or data_source_name=''))
BEGIN
	UPDATE objects_list_kdk SET filepro = NEW.filepro, data_source_name=NEW.data_source_name WHERE path=NEW.path and sm_status=NEW.sm_status and (data_source_name=NEW.data_source_name or data_source_name='');
END;

--全量数据：文件保护功能白名单视图插入触发器，当记录不存在时，在表中插入记录
CREATE TRIGGER filepro_whlist_kdk_insert_not_exists
INSTEAD OF INSERT ON filepro_whlist_kdk WHEN NOT EXISTS(select 1 from objects_list_kdk where path=NEW.path and sm_status=NEW.sm_status and (data_source_name=NEW.data_source_name or data_source_name=''))
BEGIN
	INSERT INTO objects_list_kdk(path, hash, type, exectl, filepro, rmmodpro, status, sm_status, data_source_name) values(NEW.path, NEW.hash, NEW.type, 'unknown', NEW.filepro, 0, 0, NEW.sm_status, NEW.data_source_name);
END;

--全量数据：内核模块保护功能白名单视图删除触发器，采用修改exectl字段为unknown，rmmodpro字段值为0替换，并修改status值为0
CREATE TRIGGER kmodpro_whlist_kdk_delete
INSTEAD OF DELETE ON kmodpro_whlist_kdk
BEGIN
	UPDATE objects_list_kdk SET exectl = 'unknown', status = 0 WHERE path=OLD.path and sm_status=OLD.sm_status and (data_source_name=OLD.data_source_name or data_source_name='');
END;

--全量数据：内核模块保护功能白名单视图插入触发器，当记录已存在时
CREATE TRIGGER kmodpro_whlist_kdk_insert_exists
INSTEAD OF INSERT ON kmodpro_whlist_kdk WHEN EXISTS(select 1 from objects_list_kdk where path=NEW.path and sm_status=NEW.sm_status and (data_source_name=NEW.data_source_name or data_source_name=''))
BEGIN
	UPDATE objects_list_kdk SET exectl = NEW.exectl, hash = NEW.hash, status = 0, data_source_name=NEW.data_source_name WHERE path=NEW.path and sm_status=NEW.sm_status and (data_source_name=NEW.data_source_name or data_source_name='');
END;

--全量数据：内核模块保护功能白名单视图插入触发器，当记录不存在时
CREATE TRIGGER kmodpro_whlist_kdk_insert_not_exists
INSTEAD OF INSERT ON kmodpro_whlist_kdk WHEN NOT EXISTS(select 1 from objects_list_kdk where path=NEW.path and sm_status=NEW.sm_status and (data_source_name=NEW.data_source_name or data_source_name=''))
BEGIN
	INSERT INTO objects_list_kdk(path, hash, type, exectl, filepro, rmmodpro, status, sm_status, data_source_name) values(NEW.path, NEW.hash, NEW.type, NEW.exectl, 'none', 0, 0, NEW.sm_status, NEW.data_source_name);
END;

--通用功能：执行控制功能白名单视图删除触发器，采用修改exectl字段值为'unknown'替换
CREATE TRIGGER exectl_whlist_delete
INSTEAD OF DELETE ON exectl_whlist
BEGIN
	UPDATE objects_list SET exectl = 'unknown', status = 3 WHERE path=OLD.path;
END;

--通用功能：执行控制功能白名单视图插入触发器，当记录已存在时，修改已存在记录exectl为新exectl状态
CREATE TRIGGER exectl_whlist_insert_exists
INSTEAD OF INSERT ON exectl_whlist WHEN EXISTS(select 1 from objects_list where path=NEW.path)
BEGIN
	UPDATE objects_list SET exectl = NEW.exectl, hash = NEW.hash, status = 0 WHERE path=NEW.path;
END;

--通用功能：执行控制功能白名单视图插入触发器，当记录不存在时，在表中插入记录
CREATE TRIGGER exectl_whlist_insert_not_exists
INSTEAD OF INSERT ON exectl_whlist WHEN NOT EXISTS(select 1 from objects_list where path=NEW.path)
BEGIN
	INSERT INTO objects_list(path, hash, type, exectl, filepro, rmmodpro, status) values(NEW.path, NEW.hash, NEW.type, NEW.exectl, 'none', 0, 0);
END;

--通用功能：文件保护功能视图删除触发器，采用修改filepro字段值为none替换
CREATE TRIGGER filepro_whlist_delete
INSTEAD OF DELETE ON filepro_whlist
BEGIN
	UPDATE objects_list SET filepro = 'none' WHERE path=OLD.path;	
END;

--通用功能：文件保护功能白名单视图插入触发器，当记录已存在时，修改已存在记录exectl为新exectl状态
CREATE TRIGGER filepro_whlist_insert_exists
INSTEAD OF INSERT ON filepro_whlist WHEN EXISTS(select 1 from objects_list where path=NEW.path)
BEGIN
	UPDATE objects_list SET filepro = NEW.filepro WHERE path=NEW.path;
END;

--通用功能：文件保护功能白名单视图插入触发器，当记录不存在时，在表中插入记录
CREATE TRIGGER filepro_whlist_insert_not_exists
INSTEAD OF INSERT ON filepro_whlist WHEN NOT EXISTS(select 1 from objects_list where path=NEW.path)
BEGIN
	INSERT INTO objects_list(path, hash, type, exectl, filepro, rmmodpro, status) values(NEW.path, NEW.hash, NEW.type, 'unknown', NEW.filepro, 0, 0);
END;

--通用功能：内核模块保护功能白名单视图删除触发器，采用修改exectl字段为unknown，rmmodpro字段值为0替换，并修改status值为0
CREATE TRIGGER kmodpro_whlist_delete
INSTEAD OF DELETE ON kmodpro_whlist
BEGIN
	UPDATE objects_list SET exectl = 'unknown', status = 0 WHERE path=OLD.path;
END;

--通用功能：内核模块保护功能白名单视图插入触发器，当记录已存在时，修改已存在记录exectl为新exectl状态， 并修改status值为0
CREATE TRIGGER kmodpro_whlist_insert_exists
INSTEAD OF INSERT ON kmodpro_whlist WHEN EXISTS(select 1 from objects_list where path=NEW.path)
BEGIN
	UPDATE objects_list SET exectl = NEW.exectl, hash = NEW.hash, status = 0 WHERE path=NEW.path;
END;

--通用功能：内核模块保护功能白名单视图插入触发器，当记录不存在时，在表中插入记录
CREATE TRIGGER kmodpro_whlist_insert_not_exists
INSTEAD OF INSERT ON kmodpro_whlist WHEN NOT EXISTS(select 1 from objects_list where path=NEW.path)
BEGIN
	INSERT INTO objects_list(path, hash, type, exectl, filepro, rmmodpro, status) values(NEW.path, NEW.hash, NEW.type, NEW.exectl, 'none', 0, 0);
END;
